app.component('tab-revisions', {
	props: ['item', 'formData', 'formDefinitions', 'saved'],
	data: function(){
		return {
			revisions: false,
			revision: false,
			diff: false,
			picked: false,
			disabled: false,
			message: false,
			messageClass: '',
			errors: {}
		}
	},
	template: `<section class="dark:bg-stone-700 dark:text-stone-200">
					<form class="inline-block w-full mb-8">
						<div class="w-full">
							<fieldset class="w-full border border-stone-200 px-4 py-8">
								<legend class="text-lg font-medium">Revisions</legend>
								<p>The revisions display all changes made by one user within the configured period of time (e.g., one day). If you restore a version, the text along with all the associated changes will be restored. The first revision in the list represents the current version of the content, highlighting all changes made since the previous revision.</p>
								<div 
									v-if="revisions"
									v-for="myrevision in revisions" 
									class="bg-stone-100 p-3 mt-3 hover:bg-stone-200"
									>
									<div class="flex w-full justify-between">
										<label class="px-2 py-1">
										  	<input 
										  		type 		= "radio" 
										  		v-model		= "picked" 
										  		:value 		= "myrevision.key"
										  		class  		= "mr-3"
										  		@click 		= selectRevision(myrevision) 
										  	/>
										  	<span>{{ myrevision.username }} ({{myrevision.date}})</span>
										</label>
										<div class="flex">
											<div class="px-2 py-1">
											    {{ myrevision.status }}
											    <span v-if="myrevision.status == 'draft'" class="rounddot bg-yellow-500"></span>
											    <span v-if="myrevision.status == 'live'" class="rounddot bg-teal-500"></span>
											</div>
											<button
												class="px-2 py-1 bg-stone-700 text-white hover:bg-stone-900"
												@click.prevent="restoreRevision(myrevision)"
												>Restore
											</button>
										</div>
									</div>
									<div v-if="revision.key == myrevision.key" class="pt-3">
										<div v-html="diff"></div>
									</div>
								</div>
								<div v-else class="bg-stone-100 p-3 mt-3">
									<p><strong>No revisions yet.</stong></p>
								</div>

								<div class="block w-full h-8 my-1">
									<transition name="fade">
										<div v-if="message" :class="messageClass" class="text-white px-3 py-1  transition duration-100">{{ $filters.translate(message) }}</div>
									</transition>
								</div>
							</fieldset>
						</div>
					</form>
				</section>`,
	mounted: function()
	{
		this.getRevisions();
	},
	methods: {
		reset()
		{
	    	this.localMessages 	= {}; 
			this.errors 		= {};
			this.saved 			= false;
			this.disabled 		= true;
			this.message 		= false;
			this.messageClass 	= 'bg-stone-50';
		},
		getRevisions()
		{			
			this.reset();

			var self = this;

			tmaxios.get('/api/v1/revisions',{
				params: {
					'itemurl': this.item.urlRelWoF
				}
			})
			.then(function (response)
			{
				self.disabled 		= false;
				self.revisions 		= response.data.revisions;

/*			    const firstRevision = Object.keys(self.revisions)[0];
			    if (firstRevision) 
			    {
					self.selectRevision();
				}
*/
			})
			.catch(function (error)
			{
				self.reset();
				
				if(error.response)
				{
					let errors = error.response.data.errors;
					let message = handleErrorMessage(error);

					if(message)
					{				
						self.messageClass 	= 'bg-rose-500';
						self.message 		= message;
					}

					if(errors)
					{
						self.errors = errors;
					}
				}
			});
		},
	    selectRevision(revision)
	    {
	    	this.revision = revision

	    	/* get changes of revision from api */
			this.reset();

			var self = this;

			tmaxios.get('/api/v1/revision',{
				params: {
					'itemurl': this.item.urlRelWoF,
					'timestamp': revision.timestamp
				}
			})
			.then(function (response)
			{
				self.disabled 		= false;
				self.diff 			= response.data.diff;
			})
			.catch(function (error)
			{
				self.reset();
				
				if(error.response)
				{
					let errors = error.response.data.errors;
					let message = handleErrorMessage(error);

					if(message)
					{				
						self.messageClass 	= 'bg-rose-500';
						self.message 		= message;
					}

					if(errors)
					{
						self.errors = errors;
					}
				}
			});
	    },
	    restoreRevision(revision)
	    {
	    	this.reset();

			var self = this;

			tmaxios.put('/api/v1/revision',{
				'url':			data.urlinfo.route,
				'item': 		this.item,
				'timestamp': 	revision.timestamp
			})
			.then(function (response){
				self.disabled 		= false;
				self.revisions 		= response.data.revisions;
				window.scrollTo({ top: 0, behavior: 'smooth' });
				window.addEventListener('scroll', function onScroll()
				{
	                if (window.scrollY === 0) {
	                    window.removeEventListener('scroll', onScroll);
	                    location.reload();
	                }
	            });				
			})
			.catch(function (error){

				self.reset();
				
				if(error.response)
				{
					let errors = error.response.data.errors;
					let message = handleErrorMessage(error);

					if(message)
					{				
						self.messageClass 	= 'bg-rose-500';
						self.message 		= message;
					}

					if(errors)
					{
						self.errors = errors;
					}
				}
			});
		}
	}
})

