app.component('tab-templates', {
	props: ['item', 'formData', 'formDefinitions', 'saved'],
	data: function(){
		return {
			languages: [],
			localtemplates: false,
			selectedLocalLang: null,
			storelanguage: '',
			storename: '',
			localMessages: {},

			remotetemplates: false,
			selectedRemoteTopic: null,
			selectedRemoteLang: null,
			remoteMessages: {},

			disabled: false,
			message: false,
			messageClass: '',
			errors: {}
		}
	},
	template: `<section class="dark:bg-stone-700 dark:text-stone-200">
					<form class="inline-block w-full mb-8">
						<div class="w-full">
							<fieldset class="w-full border border-stone-200 px-4 py-8">
								<legend class="text-lg font-medium">Store Page as Template</legend>
								<div class="flex flex-wrap justify-between">
									<div class="lg:w-half w-full mt-5 mb-2">
										<label class="block mb-1 font-medium">Template Name</label>
										<input v-model="storename" class="text-stone-900 h-12 w-full border px-2 py-3 border-stone-300 bg-stone-200" type="text" pattern="[a-z0-9]">
									</div>
									<div class="lg:w-half w-full mt-5 mb-2">
										<label class="block mb-1 font-medium">Template Language</label>
										<select v-model="storelanguage" class="form-select block w-full border border-stone-300 text-stone-900 bg-stone-200 px-2 py-3 h-12 transition ease-in-out border-stone-300 bg-stone-200">
											<option v-for="lang in languages">{{lang}}</option>
										</select>
									</div>
								</div>
								<div class="my-2">
									<div class="block w-full h-8 my-1">
										<transition name="fade">
											<div v-if="message" :class="messageClass" class="text-white px-3 py-1  transition duration-100">{{ $filters.translate(message) }}</div>
										</transition>
									</div>
									<input 
										type 			= "submit" 
										@click.prevent 	= "storeLocalTemplate()" 
										:value 			= "$filters.translate('save')" 
										class 			= "w-full p-3 my-1 bg-stone-700 dark:bg-stone-600 hover:bg-stone-900 hover:dark:bg-stone-900 text-white cursor-pointer transition duration-100"
									>
								</div>
							</fieldset>
						</div>
						<div v-if="localtemplates" class="w-full">
							<fieldset class="border-2 border-stone-200 p-4 my-8">
								<legend class="text-lg font-medium">Your Templates</legend>

							    <div class="w-full">
									<button 
								        v-for			= "(template, lang) in localtemplates"
								        :key			= "index"
								        @click.prevent	= "selectLocalLang(lang)"
										class 			= "px-2 py-2 border-b-2 border-white dark:border-stone-900 hover:border-stone-700 hover:dark:border-stone-200 hover:bg-stone-100 hover:dark:bg-stone-900 transition duration-100" 
										:class 			= "(lang == selectedLocalLang) ? 'border-stone-700 bg-stone-100 dark:bg-stone-900 dark:border-stone-200' : ''" 
								      >
								        {{ lang }}
								      </button>
							    </div>

							    <div v-if="selectedLocalLang !== null" class="w-full">
							        <ul class="border border-stone-200">
							          <li 
							          	v-for 	= "(templatename,index) in localtemplates[selectedLocalLang]" 
							          	:key 	= "templatename"
							          	class 	= "flex justify-between bg-stone-100 hover:bg-stone-200 px-1 py-1"
							          	>
							          		<div class="flex px-2 py-2">
												<span v-if="isfile(templatename)" class="mr-3 text-stone-500">
													<svg class="h-5 w-5" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 28">
														<path fill="currentColor" d="M22.937 5.938c0.578 0.578 1.062 1.734 1.062 2.562v18c0 0.828-0.672 1.5-1.5 1.5h-21c-0.828 0-1.5-0.672-1.5-1.5v-25c0-0.828 0.672-1.5 1.5-1.5h14c0.828 0 1.984 0.484 2.562 1.062zM16 2.125v5.875h5.875c-0.094-0.266-0.234-0.531-0.344-0.641l-4.891-4.891c-0.109-0.109-0.375-0.25-0.641-0.344zM22 26v-16h-6.5c-0.828 0-1.5-0.672-1.5-1.5v-6.5h-12v24h20zM6 12.5c0-0.281 0.219-0.5 0.5-0.5h11c0.281 0 0.5 0.219 0.5 0.5v1c0 0.281-0.219 0.5-0.5 0.5h-11c-0.281 0-0.5-0.219-0.5-0.5v-1zM17.5 16c0.281 0 0.5 0.219 0.5 0.5v1c0 0.281-0.219 0.5-0.5 0.5h-11c-0.281 0-0.5-0.219-0.5-0.5v-1c0-0.281 0.219-0.5 0.5-0.5h11zM17.5 20c0.281 0 0.5 0.219 0.5 0.5v1c0 0.281-0.219 0.5-0.5 0.5h-11c-0.281 0-0.5-0.219-0.5-0.5v-1c0-0.281 0.219-0.5 0.5-0.5h11z"></path>
													</svg>
												</span>
												<span v-else class="mr-3 text-stone-500">
													<svg class="h-5 w-5" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 28">
														<path fill="currentColor" d="M24 20.5v-11c0-0.828-0.672-1.5-1.5-1.5h-11c-0.828 0-1.5-0.672-1.5-1.5v-1c0-0.828-0.672-1.5-1.5-1.5h-5c-0.828 0-1.5 0.672-1.5 1.5v15c0 0.828 0.672 1.5 1.5 1.5h19c0.828 0 1.5-0.672 1.5-1.5zM26 9.5v11c0 1.922-1.578 3.5-3.5 3.5h-19c-1.922 0-3.5-1.578-3.5-3.5v-15c0-1.922 1.578-3.5 3.5-3.5h5c1.922 0 3.5 1.578 3.5 3.5v0.5h10.5c1.922 0 3.5 1.578 3.5 3.5z"></path>
													</svg>
												</span>
								          		<span> 
								          			{{ templatename }}
								          		</span>
							          		</div>
							          		<div v-if="getLocalMessage(templatename)" class="px-2 py-2">{{ getLocalMessage(templatename)}}</div>
							          		<div>
							          			<button
							          				v-if 			= "checkLocalOverwrite(templatename)"
							          				class 			= "px-2 py-2 mr-1 text-white bg-yellow-500 hover:bg-yellow-600"
							          				@click.prevent 	= "loadLocalTemplate(templatename, true)"
							          				>overwrite</button>
							          			<button 
							          				class 			= "px-2 py-2 mr-1 text-white bg-stone-700 hover:bg-stone-900"
							          				@click.prevent 	= "loadLocalTemplate(templatename)"
							          				>load</button>
							          			<button 
							          				class = "px-2 py-2 text-white bg-rose-500 hover:bg-rose-600"
							          				@click.prevent = "deleteLocalTemplate(templatename)"
							          				>delete</button>
							          		</div>
							          </li>
							        </ul>
							    </div>
							</fieldset>
						</div>

						<div v-if="remotetemplates" class="w-full">
							<fieldset class="border-2 border-stone-200 p-4 my-8">
						    	<legend class="text-lg font-medium">Global Templates</legend>

						    	<div class="w-full">
						        	<button
						          		v-for 			= "(languages, topic) in remotetemplates"
						          		:key 			= "topic"
						         		@click.prevent 	= "selectRemoteTopic(topic)"
										class 			= "px-2 py-2 border-b-2 border-white dark:border-stone-900 hover:border-stone-700 hover:dark:border-stone-200 hover:bg-stone-100 hover:dark:bg-stone-900 transition duration-100" 
										:class 			= "(topic == selectedRemoteTopic) ? 'border-stone-700 bg-stone-100 dark:bg-stone-900 dark:border-stone-200' : ''" 
						        	>
						          		{{ topic }}
						        	</button>
						     	</div>

						      	<div v-if="selectedRemoteTopic !== null" class="w-full">
						        	<div class="flex space-x-4">
						          		<button
						            		v-for 			= "(templates, language) in remotetemplates[selectedRemoteTopic]"
						            		:key 			= "language"
						            		@click.prevent 	= "selectRemoteLang(language)"
											class 			= "px-2 py-2 border-b-2 border-white dark:border-stone-900 hover:border-stone-700 hover:dark:border-stone-200 hover:bg-stone-100 hover:dark:bg-stone-900 transition duration-100" 
											:class 			= "(language == selectedRemoteLang) ? 'border-stone-700 bg-stone-100 dark:bg-stone-900 dark:border-stone-200' : ''" 
						          		>
						            		{{ language }}
						          		</button>
						        	</div>

						        	<div v-if="selectedRemoteLang !== null" class="w-full">
						          		<ul class="border border-stone-200">
						              		<li
						                		v-for 		= "templatename in remotetemplates[selectedRemoteTopic][selectedRemoteLang]"
						                		:key 		= "templatename"
						                		class 		= "flex justify-between bg-stone-100 hover:bg-stone-200 px-1 py-1"
						              		>
						                		<div class="flex px-2 py-2">
						                  			<span v-if="isfile(templatename)" class="mr-3 text-stone-500">
						                    			<svg class="h-5 w-5" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 28">
						                      				<path fill="currentColor" d="M22.937 5.938c0.578 0.578 1.062 1.734 1.062 2.562v18c0 0.828-0.672 1.5-1.5 1.5h-21c-0.828 0-1.5-0.672-1.5-1.5v-25c0-0.828 0.672-1.5 1.5-1.5h14c0.828 0 1.984 0.484 2.562 1.062zM16 2.125v5.875h5.875c-0.094-0.266-0.234-0.531-0.344-0.641l-4.891-4.891c-0.109-0.109-0.375-0.25-0.641-0.344zM22 26v-16h-6.5c-0.828 0-1.5-0.672-1.5-1.5v-6.5h-12v24h20zM6 12.5c0-0.281 0.219-0.5 0.5-0.5h11c0.281 0 0.5 0.219 0.5 0.5v1c0 0.281-0.219 0.5-0.5 0.5h-11c-0.281 0-0.5-0.219-0.5-0.5v-1zM17.5 16c0.281 0 0.5 0.219 0.5 0.5v1c0 0.281-0.219 0.5-0.5 0.5h-11c-0.281 0-0.5-0.219-0.5-0.5v-1c0-0.281 0.219-0.5 0.5-0.5h11zM17.5 20c0.281 0 0.5 0.219 0.5 0.5v1c0 0.281-0.219 0.5-0.5 0.5h-11c-0.281 0-0.5-0.219-0.5-0.5v-1c0-0.281 0.219-0.5 0.5-0.5h11z"></path>
						                    			</svg>
						                  			</span>
						                  			<span v-else class="mr-3 text-stone-500">
						                    			<svg class="h-5 w-5" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 28">
						                      				<path fill="currentColor" d="M24 20.5v-11c0-0.828-0.672-1.5-1.5-1.5h-11c-0.828 0-1.5-0.672-1.5-1.5v-1c0-0.828-0.672-1.5-1.5-1.5h-5c-0.828 0-1.5 0.672-1.5 1.5v15c0 0.828 0.672 1.5 1.5 1.5h19c0.828 0 1.5-0.672 1.5-1.5zM26 9.5v11c0 1.922-1.578 3.5-3.5 3.5h-19c-1.922 0-3.5-1.578-3.5-3.5v-15c0-1.922 1.578-3.5 3.5-3.5h5c1.922 0 3.5 1.578 3.5 3.5v0.5h10.5c1.922 0 3.5 1.578 3.5 3.5z"></path>
						                    			</svg>
						                  			</span>
						                  			<span>
						                    			{{ templatename }}
						                  			</span>
						                		</div>
						                		<div v-if="getRemoteMessage(templatename)" class="px-2 py-2">{{ getRemoteMessage(templatename)}}</div>
					                			<div>
					                  				<button
					                    				v-if 			= "checkRemoteOverwrite(templatename)"
					                    				class 			= "px-2 py-2 mr-1 text-white bg-yellow-500 hover:bg-yellow-600"
					                    				@click.prevent 	= "loadRemoteTemplate(templatename, true)"
					                  				>overwrite</button>
					                  				<button
					                    				class 			= "px-2 py-2 mr-1 text-white bg-stone-700 hover:bg-stone-900"
					                    				@click.prevent 	= "loadRemoteTemplate(templatename)"
					                  				>load</button>
					                			</div>
					              			</li>
						          		</ul>
						        	</div>
						      	</div>
						    </fieldset>
						</div>
					</form>
				</section>`,
	mounted: function()
	{
		if(data.settings.langattr != undefined)
		{
			this.languages.push(data.settings.langattr);
		}

		this.getLocalTemplates();

		this.getRemoteTemplates();
	},
	methods: {
		reset()
		{
	    	this.localMessages 	= {}; 
			this.errors 		= {};
			this.saved 			= false;
			this.disabled 		= true;
			this.message 		= false;
			this.messageClass 	= 'bg-stone-50';
		},
	    selectRemoteTopic(topic) {
	      this.selectedRemoteTopic = topic;
	    },
	    selectRemoteLang(lang) {
	      this.selectedRemoteLang = lang;
	    },
	    selectLocalLang(lang)
	    {
	      this.selectedLocalLang = lang;
	    },
	    isfile(item)
	   	{
	   		return item.endsWith('.md');
	   	},
		checkLocalOverwrite(templatename)
		{
		    if (this.localMessages[templatename] && this.localMessages[templatename].overwrite !== undefined)
		    {
		        return true;
		    }
		    return false;
		},
		checkRemoteOverwrite(templatename)
		{
		    if (this.remoteMessages[templatename] && this.remoteMessages[templatename].overwrite !== undefined)
		    {
		        return true;
		    }
		    return false;
		},
		getLocalMessage(templatename)
		{
		    if (this.localMessages[templatename] && this.localMessages[templatename].message !== undefined)
		    {
		        return this.localMessages[templatename].message;
		    }
		    return false;
		},
		getRemoteMessage(templatename)
		{
		    if (this.remoteMessages[templatename] && this.remoteMessages[templatename].message !== undefined)
		    {
		        return this.remoteMessages[templatename].message;
		    }
		    return false;
		},	   	
		getLocalTemplates: function()
		{
			this.reset();

			var self = this;

			tmaxios.get('/api/v1/localtemplates',{
				params: {
					'url': data.urlinfo.route
				}
			})
			.then(function (response)
			{
				self.disabled 			= false;
				self.localtemplates 	= response.data.templates;

			    const firstTab = Object.keys(self.localtemplates)[0];
			    if (firstTab) 
			    {
					self.selectLocalLang(firstTab);
				}

				for (const language in self.localtemplates)
				{
				    if (self.localtemplates.hasOwnProperty(language) && !self.languages.includes(language))
				    {
				        self.languages.push(language);
				    }
				}
			})
			.catch(function (error)
			{
				self.reset();
				
				if(error.response)
				{
					let errors = error.response.data.errors;
					let message = handleErrorMessage(error);

					if(message)
					{				
						self.messageClass 	= 'bg-rose-500';
						self.message 		= message;
					}

					if(errors)
					{
						self.errors = errors;
					}
				}
			});
		},
	    storeLocalTemplate()
	    {
	    	this.reset();

			var self = this;

			tmaxios.post('/api/v1/localtemplate',{
				'url':			data.urlinfo.route,
				'name':  		this.storename, 
				'language': 	this.storelanguage,
				'item': 		this.item
			})
			.then(function (response){
				self.disabled 			= false;
				self.saved 				= true;
				self.message 			= 'saved successfully';
				self.messageClass 		= 'bg-teal-500';
				self.localtemplates 	= response.data.templates;
			})
			.catch(function (error){

				self.reset();
				
				if(error.response)
				{
					let errors = error.response.data.errors;
					let message = handleErrorMessage(error);

					if(message)
					{				
						self.messageClass 	= 'bg-rose-500';
						self.message 		= message;
					}

					if(errors)
					{
						self.errors = errors;
					}
				}
			});
		},
	    deleteLocalTemplate(templatename)
	    {
	    	this.reset();

			var self = this;

			tmaxios.delete('/api/v1/localtemplate',{
				data: {
					'url':			data.urlinfo.route,
					'name':  		templatename, 
					'language': 	this.selectedLocalLang,
					'item': 		this.item
				}
			})
			.then(function (response){
				self.disabled 			= false;
				self.saved 				= true;
				self.localtemplates 	= response.data.templates;
			})
			.catch(function (error){

				self.disabled = false;
				
				if(error.response)
				{
					let errors = error.response.data.errors;
					let message = handleErrorMessage(error);

					if(message)
					{				
						self.messageClass 	= 'bg-rose-500';
						self.message 		= message;
					}

					if(errors)
					{
						self.errors = errors;
					}
				}
			});
		},
	    loadLocalTemplate(templatename, overwrite = false)
	    {
	    	this.reset();

			var self = this;

			tmaxios.put('/api/v1/localtemplate',{
				'url':			data.urlinfo.route,
				'name':  		templatename, 
				'language': 	this.selectedLocalLang,
				'item': 		this.item,
				'overwrite': 	overwrite
			})
			.then(function (response){
				self.disabled 			= false;
				self.saved 				= true;

				if(response.data.refresh)
				{
					window.scrollTo({ top: 0, behavior: 'smooth' });
					window.addEventListener('scroll', function onScroll() {
		                if (window.scrollY === 0) {
		                    window.removeEventListener('scroll', onScroll);
		                    location.reload();
		                }
		            });					
				}
			})
			.catch(function (error){

				self.disabled = false;
				
				if(error.response)
				{
					console.info(templatename);

					if(error.response.data['draft'])
					{
					    if (!self.localMessages[templatename])
					    {
					        self.localMessages[templatename] = {};
					    }
						self.localMessages[templatename].overwrite = true;
					}
					if(error.response.data['message'])
					{
					    if (!self.localMessages[templatename])
					    {
					        self.localMessages[templatename] = {};
					    }
						self.localMessages[templatename].message = error.response.data['message'];
					}
				}
			});
		},
		getRemoteTemplates: function()
		{
			this.reset();

			var self = this;

			tmaxios.get('/api/v1/remotetemplates',{
				params: {
					'url': data.urlinfo.route
				}
			})
			.then(function (response)
			{
				self.disabled 			= false;
				self.remotetemplates 	= response.data.templates;

			    const topicTab = Object.keys(self.remotetemplates)[0];
			    if (topicTab)
			    {
			        self.selectRemoteTopic(topicTab);
			        const languageTab = Object.keys(self.remotetemplates[topicTab])[0];
			        if (languageTab)
			        {
			        	self.selectedRemoteLang = languageTab;
			        }
			      }
			})
			.catch(function (error)
			{
				self.disabled 			= false;

				if(error.response)
				{
					if(error.response.data.serviceaccount != undefined)
					{
						selfseo.serviceaccount = error.response.data.serviceaccount;
					}
					if(error.response.data.gscdata != undefined)
					{
						selfseo.gscdata = error.response.data.gscdata;
					}

					let message = handleErrorMessage(error);
					if(message)
					{
						selfseo.serviceaccountmessage = message;
					}
				}
			});
		},
	    loadRemoteTemplate(templatename, overwrite = false)
	    {
	    	this.reset();

			var self = this;

			tmaxios.put('/api/v1/remotetemplate',{
				'url':			data.urlinfo.route,
				'name':  		templatename, 
				'language': 	this.selectedRemoteLang,
				'topic': 		this.selectedRemoteTopic,
				'item': 		this.item,
				'overwrite': 	overwrite
			})
			.then(function (response){
				self.disabled 			= false;
				self.saved 				= true;

				if(response.data.refresh)
				{
					window.scrollTo({ top: 0, behavior: 'smooth' });
					window.addEventListener('scroll', function onScroll() {
		                if (window.scrollY === 0) {
		                    window.removeEventListener('scroll', onScroll);
		                    location.reload();
		                }
		            });					
				}
			})
			.catch(function (error){

				self.disabled = false;
				
				if(error.response)
				{
					if(error.response.data['draft'])
					{
					    if (!self.remoteMessages[templatename])
					    {
					        self.remoteMessages[templatename] = {};
					    }
						self.remoteMessages[templatename].overwrite = true;
					}
					if(error.response.data['message'])
					{
					    if (!self.remoteMessages[templatename])
					    {
					        self.remoteMessages[templatename] = {};
					    }
						self.remoteMessages[templatename].message = error.response.data['message'];
					}
				}
			});
		},
	}
})

