<?php

namespace Plugins\Templates;

use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use \Typemill\Plugin;
use \Typemill\Extensions\ParsedownExtension;
use \Typemill\Models\Content;
use \Typemill\Models\Navigation;
use \Typemill\Models\StorageWrapper;
use \Typemill\Models\Validation;
use \Typemill\Models\License;

class Templates extends Plugin
{
	protected $item;

	protected $settings;

	protected $storage;

	protected $error;
	
	public static function setPremiumLicence()
	{
		return 'BUSINESS';
	}	

    public static function getSubscribedEvents()
    {
		return array(
			'onTwigLoaded' 		=> 'onTwigLoaded',
		);
	}
	
	# route: delete local template file
	# route: delete local template folder
	public static function addNewRoutes()
	{
		return [
			[
				'httpMethod' 	=> 'get', 
				'route' 		=> '/api/v1/localtemplates', 
				'name' 			=> 'localtemplates.get', 
				'class' 		=> 'Plugins\templates\templates:getLocalTemplates', 
				'resource' 		=> 'mycontent', 
				'privilege' 	=> 'write'
			],
			[
				'httpMethod' 	=> 'post', 
				'route' 		=> '/api/v1/localtemplate',
				'name' 			=> 'localtemplate.post', 
				'class' 		=> 'Plugins\templates\templates:storeLocalTemplate', 
				'resource' 		=> 'mycontent', 
				'privilege' 	=> 'write'
			],
			[
				'httpMethod' 	=> 'delete', 
				'route' 		=> '/api/v1/localtemplate',
				'name' 			=> 'localtemplate.delete', 
				'class' 		=> 'Plugins\templates\templates:deleteLocalTemplate', 
				'resource' 		=> 'mycontent', 
				'privilege' 	=> 'write'
			],
			[
				'httpMethod' 	=> 'put', 
				'route' 		=> '/api/v1/localtemplate',
				'name' 			=> 'localtemplate.use', 
				'class' 		=> 'Plugins\templates\templates:useLocalTemplate', 
				'resource' 		=> 'mycontent', 
				'privilege' 	=> 'write'
			],
			[
				'httpMethod' 	=> 'get', 
				'route' 		=> '/api/v1/remotetemplates', 
				'name' 			=> 'remotetemplates.get', 
				'class' 		=> 'Plugins\templates\templates:getRemoteTemplates', 
				'resource' 		=> 'mycontent', 
				'privilege' 	=> 'write'
			],
			[
				'httpMethod' 	=> 'put', 
				'route' 		=> '/api/v1/remotetemplate',
				'name' 			=> 'remotetemplate.use', 
				'class' 		=> 'Plugins\templates\templates:useRemoteTemplate', 
				'resource' 		=> 'mycontent', 
				'privilege' 	=> 'write'
			],
		];
	}

	public function onTwigLoaded($noidea)
	{
#		$this->settings = $this->getSettings();

#		$this->storage = new StorageWrapper($this->settings['storage']);

    	if($this->editorroute)
    	{
	        $this->addJS('/templates/js/templates.js');
    	}
	}

	public function getLocalTemplates(Request $request, Response $response, $args)
	{
		$settings 		= $this->getSettings();

		$storage 		= new StorageWrapper($settings['storage']);

		$folderPath 	= $storage->getFolderPath('dataFolder', 'templates');

		$folderContent 	= $this->scanLocalTemplateFolder($folderPath);

#		$templateList 	= $this->transformArrayStructure($folderContent);

		$response->getBody()->write(json_encode([
			'templates' => $folderContent
		]));

		return $response->withHeader('Content-Type', 'application/json');		
	}

	private function scanLocalTemplateFolder($folderPath, $contentTree = [])
	{
	    # Use a safer function to get folder contents
	    $folderContent = array_diff(scandir($folderPath), array('..', '.'));

	    foreach ($folderContent as $item)
	    {
	        $itemPath = realpath($folderPath . DIRECTORY_SEPARATOR . $item);

	        # Ensure $itemPath is within $folderPath to prevent directory traversal attacks
	        if (strpos($itemPath, realpath($folderPath)) !== 0)
	        {
	            continue;
	        }

	        if (is_dir($itemPath))
	        {
	            $contentTree[$item] = $this->scanLocalTemplateFolder($itemPath);
	        } 
	        else 
	        {
	            # Only process markdown files
	            if (pathinfo($itemPath, PATHINFO_EXTENSION) === 'md') 
	            {
	                # Remove the extension
#	                $contentTree[] = pathinfo($itemPath, PATHINFO_FILENAME);
	                $contentTree[] = pathinfo($itemPath, PATHINFO_BASENAME);
	            }
	        }
	    }

	    return $contentTree;
	}

	private function transformArrayStructure($array)
	{
	    $transformedArray = [];

	    foreach ($array as $topic => $languages)
	    {
	        foreach ($languages as $language => $content)
	        {
	            if (!empty($content))
	            {
	                if (!isset($transformedArray[$language]))
	                {
	                    $transformedArray[$language] = [];
	                }
	                
	                $transformedArray[$language][$topic] = $content;
	            }
	        }
	    }

	    return $transformedArray;
	}

	public function storeLocalTemplate(Request $request, Response $response, $args)
	{
		$params 		= $request->getParsedBody();
		
		$item 			= $params['item'] ?? false;
		$language 		= $params['language'] ?? false;
		$name 			= $params['name'] ?? false;
		$settings 		= $this->getSettings();

		$data 			= ['item' => $item, 'language' => $language, 'name' => $name];

		# create validation object
		$validation		= new Validation();

		# return standard valitron object for standardfields
		$v = $validation->returnValidator($data);

		$v->rule('required', ['language','name', 'item']);
		$v->rule('slug', 'name');
		$v->rule('lengthBetween', 'name', 2, 40)->message("Length between 5 - 65");
		$v->rule('regex', 'language', '/^[a-z]{2}$/')->message("Must be a-z and 2 characters");
		$v->rule('array', 'item')->message("Must be an array");
		$v->rule('array', 'item')->message("Must be an array");
		$v->rule('arrayHasKeys', 'item', ['pathWithoutType']);		
		if(!$v->validate())
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'Please correct the errors in the form.',
				'errors' 	=> $v->errors(),
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		$storage 		= new StorageWrapper($settings['storage']);

		if($storage->checkFile('dataFolder', 'templates' . DIRECTORY_SEPARATOR . $language, $name))
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'There is already a template with that name.',
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		# get markdown
		$filetype 		= '.txt';

		$markdown 		= $storage->getFile('contentFolder', '', $item['pathWithoutType'] . $filetype);

		if(!$markdown)
		{
			$filetype 	= '.md';
			$markdown 	= $storage->getFile('contentFolder', '', $item['pathWithoutType'] . $filetype);
		}

		if($filetype == '.txt')
		{
			$markdownArray 	= json_decode($markdown);
			$base_url		= $this->urlinfo['baseurl'];
			$parsedown 		= new ParsedownExtension($base_url);
			$markdown 		= $parsedown->arrayBlocksToMarkdown($markdownArray);
		}

		if(!$markdown OR $markdown == '')
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'There is no markdown content for the template.',
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		if(!$storage->checkFolder('dataFolder', 'templates' . DIRECTORY_SEPARATOR . $language))
		{
			if(!$storage->createFolder('dataFolder', 'templates' . DIRECTORY_SEPARATOR . $language))
			{
				$response->getBody()->write(json_encode([
					'message' 	=> 'Could not create the language folder for the template.',
				]));

				return $response->withHeader('Content-Type', 'application/json')->withStatus(422);				
			}
		}

		if($storage->checkFile('dataFolder', 'templates' . DIRECTORY_SEPARATOR . $language, $name . '.md'))
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'There is already a template with that name.',
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		# store file
		if(!$storage->writeFile('dataFolder', 'templates' . DIRECTORY_SEPARATOR . $language, $name . '.md', $markdown))
		{
			$response->getBody()->write(json_encode([
				'message' 	=> $storage->getError(),
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);			
		}	

		# get local templates
		$folderPath 	= $storage->getFolderPath('dataFolder', 'templates');

		$folderContent 	= $this->scanLocalTemplateFolder($folderPath);

		$response->getBody()->write(json_encode([
			'message' 	=> 'Template stored successfully.',
			'templates' => $folderContent
		]));

		return $response->withHeader('Content-Type', 'application/json');		
	}

	public function deleteLocalTemplate(Request $request, Response $response, $args)
	{
		$params 		= $request->getParsedBody();
		
		$item 			= $params['item'] ?? false;
		$language 		= $params['language'] ?? false;
		$name 			= $params['name'] ?? false;
		$settings 		= $this->getSettings();

		$data 			= ['item' => $item, 'language' => $language, 'name' => $name];

		# create validation object
		$validation		= new Validation();

		# return standard valitron object for standardfields
		$v = $validation->returnValidator($data);

		$v->rule('required', ['language','name', 'item']);
		$v->rule('regex', 'name', '/^[a-z\-\._]{2,40}$/')->message("Must be a-z-_. between 2 and 40 charackters");
		$v->rule('regex', 'language', '/^[a-z]{2}$/')->message("Must be a-z and 2 characters");
		$v->rule('array', 'item')->message("Must be an array");
		$v->rule('array', 'item')->message("Must be an array");
		$v->rule('arrayHasKeys', 'item', ['pathWithoutType']);		
		if(!$v->validate())
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'Please correct the errors in the form.',
				'errors' 	=> $v->errors(),
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		# get the metadata from page
		$storage 		= new StorageWrapper($settings['storage']);

		if(!$storage->checkFile('dataFolder', 'templates' . DIRECTORY_SEPARATOR . $language, $name))
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'We did not find a template with that name.',
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		if(!$storage->deleteFile('dataFolder', 'templates' . DIRECTORY_SEPARATOR . $language, $name))
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'Error: ' . $storage->getError(),
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		# get local templates
		$folderPath 	= $storage->getFolderPath('dataFolder', 'templates');

		$folderContent 	= $this->scanLocalTemplateFolder($folderPath);

		$response->getBody()->write(json_encode([
			'message' 	=> 'Template deleted successfully.',
			'templates' => $folderContent
		]));

		return $response->withHeader('Content-Type', 'application/json');
	}

	public function useLocalTemplate(Request $request, Response $response, $args)
	{
		$params 		= $request->getParsedBody();
		
		$item 			= $params['item'] ?? false;
		$language 		= $params['language'] ?? false;
		$name 			= $params['name'] ?? false;
		$overwrite 		= $params['overwrite'] ?? false;
		$settings 		= $this->getSettings();

		$data 			= ['item' => $item, 'language' => $language, 'name' => $name];

		# create validation object
		$validation		= new Validation();

		# return standard valitron object for standardfields
		$v = $validation->returnValidator($data);

		$v->rule('required', ['language','name', 'item']);
		$v->rule('regex', 'name', '/^[a-z\-\._]{2,40}$/')->message("Must be a-z-_. between 2 and 40 charackters");
		$v->rule('regex', 'language', '/^[a-z]{2}$/')->message("Must be a-z and 2 characters");
		$v->rule('array', 'item')->message("Must be an array");
		$v->rule('arrayHasKeys', 'item', ['pathWithoutType']);		
		if(!$v->validate())
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'Please correct the errors in the form.',
				'errors' 	=> $v->errors(),
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

	    $item = (object) $item;

		$storage 		= new StorageWrapper($settings['storage']);

		# check if txt file in content folder is there
		if(isset($overwrite) && $overwrite !== true)
		{
			if($storage->checkFile('contentFolder', '', $item->pathWithoutType . '.txt'))
			{
				$response->getBody()->write(json_encode([
					'message' 	=> 'Overwrite existing draft?',
					'draft'		=> true
				]));

				return $response->withHeader('Content-Type', 'application/json')->withStatus(422);			
			}
		}

		$markdown 		= $storage->getFile('dataFolder', 'templates' . DIRECTORY_SEPARATOR . $language, $name);

		if($markdown === false)
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'We did not find a template with that name.' . $storage->getError(),
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		$content 		= new Content();

		$markdownArray = $content->markdownTextToArray($markdown);

		$content->saveDraftMarkdown($item, $markdownArray);

		$navigation 	= new Navigation();
		$navigation->clearNavigation();

		$response->getBody()->write(json_encode([
			'message' 	=> 'Template installed successfully.',
			'refresh' 	=> true
		]));

		return $response->withHeader('Content-Type', 'application/json');
	}

	public function getRemoteTemplates(Request $request, Response $response, $args)
	{
		$license 		= new License();

		$token 			= $license->getToken();

		if(!$token)
		{
			$response->getBody()->write(json_encode([
				'message' 	=> $license->getMessage(),
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);			
		}

		$url 	= 'https://service.typemill.net/api/v1/service/templates';

		if(in_array('curl', get_loaded_extensions()))
		{
	        $curl = curl_init($url);
	        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
	        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
	            "Accept: application/json",
	            "Authorization: Bearer $token",
	            "Connection: close"
	        ));

			$serviceResponse = curl_exec($curl);

			if (curl_errno($curl))
			{
				$this->message 	= Translations::translate('We got a curl error: ') . curl_error($curl);

				return false;
			}

			curl_close($curl);
		}
		else
		{

	        $options = array(
	            'http' => array(
	                'method' => 'GET',
	                'ignore_errors' => true,
	                'header' => "Accept: application/json\r\n" .
	                            "Authorization: Bearer $token\r\n" .
	                            "Connection: close\r\n",
	            )
	        );

			$context = stream_context_create($options);

			$serviceResponse = file_get_contents($url, false, $context);

			if ($serviceResponse === FALSE)
			{
			    if (!empty($http_response_header))
			    {
			        list($version, $status_code, $msg) = explode(' ', $http_response_header[0], 3);
			        
					$this->message 	= Translations::translate('We got an error from file_get_contents: ') . $status_code . ' ' . $msg;
			    }
			    else
			    {
					$this->message 	= Translations::translate('No HTTP response received or file_get_contents is blocked.');
			    }

				return false;
			}
		}

		$data = json_decode($serviceResponse,true);

		if(!isset($data['templates']))
		{
			$response->getBody()->write(json_encode([
				'message' 	=> "no templates returned",
				'data'		=> $data
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);			
		}

		$response->getBody()->write(json_encode([
			'templates' 	=> $data['templates'],
		]));

		return $response->withHeader('Content-Type', 'application/json');

	}

	public function useRemoteTemplate(Request $request, Response $response, $args)
	{
		$params 		= $request->getParsedBody();
		
		$item 			= $params['item'] ?? false;
		$language 		= $params['language'] ?? false;
		$topic 			= $params['topic'] ?? false;
		$name 			= $params['name'] ?? false;
		$overwrite 		= $params['overwrite'] ?? false;
		$settings 		= $this->getSettings();

		$data 			= ['item' => $item, 'language' => $language, 'topic' => $topic, 'name' => $name];

		# create validation object
		$validation		= new Validation();

		# return standard valitron object for standardfields
		$v = $validation->returnValidator($data);

		$v->rule('required', ['language', 'topic', 'name', 'item']);
		$v->rule('regex', 'name', '/^[a-z\-\._]{2,40}$/')->message("Must be a-z-_. between 2 and 40 charackters");
		$v->rule('regex', 'language', '/^[a-z]{2}$/')->message("Must be a-z and 2 characters");
		$v->rule('regex', 'topic', '/^[a-z\-\._]{2,40}$/')->message("Must be a-z-_. between 2 and 40 charackters");
		$v->rule('array', 'item')->message("Must be an array");
		$v->rule('arrayHasKeys', 'item', ['pathWithoutType']);		
		if(!$v->validate())
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'Please correct the errors in the form.',
				'errors' 	=> $v->errors(),
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

	    $item = (object) $item;

		$storage 		= new StorageWrapper($settings['storage']);

		# check if txt file in content folder is there
		if(isset($overwrite) && $overwrite !== true)
		{
			if($storage->checkFile('contentFolder', '', $item->pathWithoutType . '.txt'))
			{
				$response->getBody()->write(json_encode([
					'message' 	=> 'Overwrite existing draft?',
					'draft'		=> true
				]));

				return $response->withHeader('Content-Type', 'application/json')->withStatus(422);			
			}
		}

		$license 		= new License();

		$token 			= $license->getToken();

		if(!$token)
		{
			$response->getBody()->write(json_encode([
				'message' 	=> $license->getMessage(),
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);			
		}

		$language 	= urlencode($language);
		$name 		= urlencode($name);
		$topic 		= urlencode($topic);
		$url 		= "https://service.typemill.net/api/v1/service/template?language={$language}&name={$name}&topic={$topic}";

		if(in_array('curl', get_loaded_extensions()))
		{
	        $curl = curl_init($url);
	        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
	        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
	            "Accept: application/json",
	            "Authorization: Bearer $token",
	            "Connection: close"
	        ));

			$serviceResponse = curl_exec($curl);

			if (curl_errno($curl))
			{
				$this->message 	= Translations::translate('We got a curl error: ') . curl_error($curl);

				return false;
			}

			curl_close($curl);
		}
		else
		{

	        $options = array(
	            'http' => array(
	                'method' => 'GET',
	                'ignore_errors' => true,
	                'header' => "Accept: application/json\r\n" .
	                            "Authorization: Bearer $token\r\n" .
	                            "Connection: close\r\n",
	            )
	        );

			$context = stream_context_create($options);

			$serviceResponse = file_get_contents($url, false, $context);

			if ($serviceResponse === FALSE)
			{
			    if (!empty($http_response_header))
			    {
			        list($version, $status_code, $msg) = explode(' ', $http_response_header[0], 3);
			        
					$this->message 	= Translations::translate('We got an error from file_get_contents: ') . $status_code . ' ' . $msg;
			    }
			    else
			    {
					$this->message 	= Translations::translate('No HTTP response received or file_get_contents is blocked.');
			    }

				return false;
			}
		}

		$data = json_decode($serviceResponse,true);

		if(!isset($data['template']))
		{
			$response->getBody()->write(json_encode([
				'message' 	=> "no templates returned" . $data,
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);			
		}

		$markdown 		= $data['template'];

		if($markdown === false || !is_string($markdown) || empty($markdown))
		{
			$response->getBody()->write(json_encode([
				'message' 	=> 'Template data is empty or not a string',
			]));

			return $response->withHeader('Content-Type', 'application/json')->withStatus(422);
		}

		$content 		= new Content();

		$markdownArray = $content->markdownTextToArray($markdown);

		$markdownArrayNoHtml = $this->sanitizeMarkdownArray($markdownArray);

		$content->saveDraftMarkdown($item, $markdownArray);

		$navigation 	= new Navigation();
		$navigation->clearNavigation();

		$response->getBody()->write(json_encode([
			'message' 	=> 'Template installed successfully.',
			'refresh' 	=> true
		]));

		return $response->withHeader('Content-Type', 'application/json');
	}

	private function sanitizeMarkdownArray($markdownArray)
	{
		$cleanMarkdownArray = [];

		foreach($markdownArray as $key => $markdown)
		{
			if($this->markdownContainsHtml($markdown))
			{
				continue;
			}

			$cleanMarkdownArray[] = $markdown;
		}

		return $cleanMarkdownArray;
	}

	private function markdownContainsHtml($markdown)
	{
	    # Strip out code blocks (```` ... ```` or ``` ... ``` or `` ... `` or ` ... `)
	    $markdown = preg_replace('/`{4,}[\s\S]+?`{4,}/', '', $markdown);
	    $markdown = preg_replace('/`{3,}[\s\S]+?`{3,}/', '', $markdown);
	    $markdown = preg_replace('/`{2,}[\s\S]+?`{2,}/', '', $markdown);
	    $markdown = preg_replace('/`{1,}[\s\S]+?`{1,}/', '', $markdown);

	    # Check block quotes
	    preg_match_all('/^ *>[\s\S]+?[\n\r]/m', $markdown, $matches);
	    foreach ($matches[0] as $match) {
	        if (strip_tags($match) !== $match) {
	            return true;
	        }
	    }

	    # Strip out block quotes
	    $markdown = preg_replace('/^ *>[\s\S]+?[\n\r]/m', '', $markdown);

	    # Check remaining content for HTML tags
	    if ($markdown !== strip_tags($markdown)) {
	        return true;
	    }

	    return false;
	}
}